import abc
import datetime
import typing

import QuantConnect.Logging
import System
import System.Collections.Concurrent

QuantConnect_Logging__EventContainer_Callable = typing.TypeVar("QuantConnect_Logging__EventContainer_Callable")
QuantConnect_Logging__EventContainer_ReturnType = typing.TypeVar("QuantConnect_Logging__EventContainer_ReturnType")


class WhoCalledMe(System.Object):
    """Provides methods for determining higher stack frames"""

    @staticmethod
    def GetMethodName(frame: int = 1) -> str:
        """
        Gets the method name of the caller
        
        :param frame: The number of stack frames to retrace from the caller's position
        :returns: The method name of the containing scope 'frame' stack frames above the caller.
        """
        ...


class ILogHandler(System.IDisposable, metaclass=abc.ABCMeta):
    """Interface for redirecting log output"""

    def Debug(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The debug text to log
        """
        ...

    def Error(self, text: str) -> None:
        """
        Write error message to log
        
        :param text: The error text to log
        """
        ...

    def Trace(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The trace text to log
        """
        ...


class ConsoleLogHandler(System.Object, QuantConnect.Logging.ILogHandler):
    """ILogHandler implementation that writes log output to console."""

    @typing.overload
    def __init__(self) -> None:
        """Initializes a new instance of the QuantConnect.Logging.ConsoleLogHandler class."""
        ...

    @typing.overload
    def __init__(self, dateFormat: str = ...) -> None:
        """
        Initializes a new instance of the QuantConnect.Logging.ConsoleLogHandler class.
        
        :param dateFormat: Specifies the date format to use when writing log messages to the console window
        """
        ...

    def Debug(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The debug text to log
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def Error(self, text: str) -> None:
        """
        Write error message to log
        
        :param text: The error text to log
        """
        ...

    def Trace(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The trace text to log
        """
        ...


class ConsoleErrorLogHandler(QuantConnect.Logging.ConsoleLogHandler):
    """Subclass of ConsoleLogHandler that only logs error messages"""

    def Debug(self, text: str) -> None:
        """
        Hide debug messages from log
        
        :param text: The debug text to log
        """
        ...

    def Trace(self, text: str) -> None:
        """
        Hide trace messages from log
        
        :param text: The trace text to log
        """
        ...


class LogType(System.Enum):
    """Error level"""

    Debug = 0
    """Debug log level"""

    Trace = 1
    """Trace log level"""

    Error = 2
    """Error log level"""


class LogEntry(System.Object):
    """Log entry wrapper to make logging simpler:"""

    @property
    def Time(self) -> datetime.datetime:
        """Time of the log entry"""
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        """Time of the log entry"""
        ...

    @property
    def Message(self) -> str:
        """Message of the log entry"""
        ...

    @Message.setter
    def Message(self, value: str):
        """Message of the log entry"""
        ...

    @property
    def MessageType(self) -> QuantConnect.Logging.LogType:
        """Descriptor of the message type."""
        ...

    @MessageType.setter
    def MessageType(self, value: QuantConnect.Logging.LogType):
        """Descriptor of the message type."""
        ...

    @typing.overload
    def __init__(self, message: str) -> None:
        """Create a default log message with the current time."""
        ...

    @typing.overload
    def __init__(self, message: str, time: typing.Union[datetime.datetime, datetime.date], type: QuantConnect.Logging.LogType = ...) -> None:
        """
        Create a log entry at a specific time in the analysis (for a backtest).
        
        :param message: Message for log
        :param time: Utc time of the message
        :param type: Type of the log entry
        """
        ...

    def ToString(self) -> str:
        """Helper override on the log entry."""
        ...


class FileLogHandler(System.Object, QuantConnect.Logging.ILogHandler):
    """Provides an implementation of ILogHandler that writes all log messages to a file on disk."""

    @typing.overload
    def __init__(self, filepath: str, useTimestampPrefix: bool = True) -> None:
        """
        Initializes a new instance of the FileLogHandler class to write messages to the specified file path.
        The file will be opened using FileMode.Append
        
        :param filepath: The file path use to save the log messages
        :param useTimestampPrefix: True to prefix each line in the log which the UTC timestamp, false otherwise
        """
        ...

    @typing.overload
    def __init__(self) -> None:
        """Initializes a new instance of the FileLogHandler class using 'log.txt' for the filepath."""
        ...

    def CreateMessage(self, text: str, level: str) -> str:
        """
        Creates the message to be logged
        
        This method is protected.
        
        :param text: The text to be logged
        :param level: The logging leel
        """
        ...

    def Debug(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The debug text to log
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def Error(self, text: str) -> None:
        """
        Write error message to log
        
        :param text: The error text to log
        """
        ...

    def Trace(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The trace text to log
        """
        ...


class RegressionFileLogHandler(QuantConnect.Logging.FileLogHandler):
    """
    Provides an implementation of ILogHandler that writes all log messages to a file on disk
    without timestamps.
    """

    def __init__(self) -> None:
        """
        Initializes a new instance of the RegressionFileLogHandler class
        that will write to a 'regression.log' file in the executing directory
        """
        ...


class Log(System.Object):
    """Logging management class."""

    LogHandler: QuantConnect.Logging.ILogHandler
    """Gets or sets the ILogHandler instance used as the global logging implementation."""

    DebuggingEnabled: bool
    """Global flag whether to enable debugging logging:"""

    FilePath: str
    """Global flag to specify file based log path"""

    DebuggingLevel: int
    """Set the minimum message level:"""

    @staticmethod
    def Debug(text: str, level: int = 1) -> None:
        """
        Output to the console
        
        :param text: The message to show
        :param level: debug level
        """
        ...

    @staticmethod
    @typing.overload
    def Error(error: str, overrideMessageFloodProtection: bool = False) -> None:
        """
        Log error
        
        :param error: String Error
        :param overrideMessageFloodProtection: Force sending a message, overriding the "do not flood" directive
        """
        ...

    @staticmethod
    @typing.overload
    def Error(exception: System.Exception, message: str = None, overrideMessageFloodProtection: bool = False) -> None:
        """
        Log error
        
        :param exception: The exception to be logged
        :param message: An optional message to be logged, if null/whitespace the messge text will be extracted
        :param overrideMessageFloodProtection: Force sending a message, overriding the "do not flood" directive
        """
        ...

    @staticmethod
    @typing.overload
    def Error(format: str, *args: typing.Any) -> None:
        """Writes the message in red"""
        ...

    @staticmethod
    @typing.overload
    def Trace(traceText: str, overrideMessageFloodProtection: bool = False) -> None:
        """Log trace"""
        ...

    @staticmethod
    @typing.overload
    def Trace(format: str, *args: typing.Any) -> None:
        """Writes the message in normal text"""
        ...

    @staticmethod
    def VarDump(obj: typing.Any, recursion: int = 0) -> str:
        """C# Equivalent of Print_r in PHP:"""
        ...


class CompositeLogHandler(System.Object, QuantConnect.Logging.ILogHandler):
    """Provides an ILogHandler implementation that composes multiple handlers"""

    @typing.overload
    def __init__(self) -> None:
        """Initializes a new instance of the CompositeLogHandler that pipes log messages to the console and log.txt"""
        ...

    @typing.overload
    def __init__(self, *handlers: QuantConnect.Logging.ILogHandler) -> None:
        """
        Initializes a new instance of the CompositeLogHandler class from the specified handlers
        
        :param handlers: The implementations to compose
        """
        ...

    def Debug(self, text: str) -> None:
        """Write debug message to log"""
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def Error(self, text: str) -> None:
        """Write error message to log"""
        ...

    def Trace(self, text: str) -> None:
        """Write debug message to log"""
        ...


class LogHandlerExtensions(System.Object):
    """Logging extensions."""

    @staticmethod
    def Debug(logHandler: QuantConnect.Logging.ILogHandler, text: str, *args: typing.Any) -> None:
        """
        Write debug message to log
        
        :param text: Message
        :param args: Arguments to format.
        """
        ...

    @staticmethod
    def Error(logHandler: QuantConnect.Logging.ILogHandler, text: str, *args: typing.Any) -> None:
        """
        Write error message to log
        
        :param text: Message
        :param args: Arguments to format.
        """
        ...

    @staticmethod
    def Trace(logHandler: QuantConnect.Logging.ILogHandler, text: str, *args: typing.Any) -> None:
        """
        Write debug message to log
        
        :param text: Message
        :param args: Arguments to format.
        """
        ...


class FunctionalLogHandler(System.Object, QuantConnect.Logging.ILogHandler):
    """ILogHandler implementation that writes log output to result handler"""

    @typing.overload
    def __init__(self) -> None:
        """Default constructor to handle MEF."""
        ...

    @typing.overload
    def __init__(self, debug: typing.Callable[[str], None], trace: typing.Callable[[str], None], error: typing.Callable[[str], None]) -> None:
        """Initializes a new instance of the QuantConnect.Logging.FunctionalLogHandler class."""
        ...

    def Debug(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The debug text to log
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def Error(self, text: str) -> None:
        """
        Write error message to log
        
        :param text: The error text to log
        """
        ...

    def Trace(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The trace text to log
        """
        ...


class QueueLogHandler(System.Object, QuantConnect.Logging.ILogHandler):
    """ILogHandler implementation that queues all logs and writes them when instructed."""

    @property
    def Logs(self) -> System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Logging.LogEntry]:
        """Public access to the queue for log processing."""
        ...

    @property
    def LogEvent(self) -> _EventContainer[typing.Callable[[QuantConnect.Logging.LogEntry], None], None]:
        """Logging Event Handler"""
        ...

    @LogEvent.setter
    def LogEvent(self, value: _EventContainer[typing.Callable[[QuantConnect.Logging.LogEntry], None], None]):
        """Logging Event Handler"""
        ...

    def __init__(self) -> None:
        """Initializes a new instance of the QueueLogHandler class."""
        ...

    def Debug(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The debug text to log
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def Error(self, text: str) -> None:
        """
        Write error message to log
        
        :param text: The error text to log
        """
        ...

    def LogEventRaised(self, log: QuantConnect.Logging.LogEntry) -> None:
        """LOgging event delegate"""
        ...

    def OnLogEvent(self, log: QuantConnect.Logging.LogEntry) -> None:
        """
        Raise a log event safely
        
        This method is protected.
        """
        ...

    def Trace(self, text: str) -> None:
        """
        Write debug message to log
        
        :param text: The trace text to log
        """
        ...


class _EventContainer(typing.Generic[QuantConnect_Logging__EventContainer_Callable, QuantConnect_Logging__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_Logging__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_Logging__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_Logging__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





import datetime
import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Brokerages.Tradier
import QuantConnect.Data
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Generic

QuantConnect_Brokerages_Tradier__EventContainer_Callable = typing.TypeVar("QuantConnect_Brokerages_Tradier__EventContainer_Callable")
QuantConnect_Brokerages_Tradier__EventContainer_ReturnType = typing.TypeVar("QuantConnect_Brokerages_Tradier__EventContainer_ReturnType")


class TokenResponse(System.Object):
    """Token response model from QuantConnect terminal"""

    @property
    def AccessToken(self) -> str:
        ...

    @AccessToken.setter
    def AccessToken(self, value: str):
        ...

    @property
    def RefreshToken(self) -> str:
        ...

    @RefreshToken.setter
    def RefreshToken(self, value: str):
        ...

    @property
    def ExpiresIn(self) -> int:
        ...

    @ExpiresIn.setter
    def ExpiresIn(self, value: int):
        ...

    @property
    def Scope(self) -> str:
        ...

    @Scope.setter
    def Scope(self, value: str):
        ...

    @property
    def IssuedAt(self) -> datetime.datetime:
        ...

    @IssuedAt.setter
    def IssuedAt(self, value: datetime.datetime):
        ...

    @property
    def Success(self) -> bool:
        ...

    @Success.setter
    def Success(self, value: bool):
        ...

    def __init__(self) -> None:
        """Default constructor:"""
        ...


class TradierBrokerageFactory(QuantConnect.Brokerages.BrokerageFactory):
    """Provides an implementations of IBrokerageFactory that produces a TradierBrokerage"""

    class Configuration(System.Object):
        """Gets tradier values from configuration"""

        UseSandbox: bool
        """Gets whether to use the developer sandbox or not"""

        AccountId: str
        """Gets the account ID to be used when instantiating a brokerage"""

        AccessToken: str
        """Gets the access token from configuration"""

    @property
    def BrokerageData(self) -> System.Collections.Generic.Dictionary[str, str]:
        """Gets the brokerage data required to run the brokerage from configuration/disk"""
        ...

    def __init__(self) -> None:
        """Initializes a new instance of he TradierBrokerageFactory class"""
        ...

    def CreateBrokerage(self, job: QuantConnect.Packets.LiveNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Interfaces.IBrokerage:
        """
        Creates a new IBrokerage instance
        
        :param job: The job packet to create the brokerage for
        :param algorithm: The algorithm instance
        :returns: A new brokerage instance.
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def GetBrokerageModel(self, orderProvider: QuantConnect.Securities.IOrderProvider) -> QuantConnect.Brokerages.IBrokerageModel:
        """
        Gets a new instance of the TradierBrokerageModel
        
        :param orderProvider: The order provider
        """
        ...


class TradierUserAccount(System.Object):
    """Account only settings for a tradier user:"""

    @property
    def AccountNumber(self) -> int:
        ...

    @AccountNumber.setter
    def AccountNumber(self, value: int):
        ...

    @property
    def DayTrader(self) -> bool:
        ...

    @DayTrader.setter
    def DayTrader(self, value: bool):
        ...

    @property
    def OptionLevel(self) -> int:
        ...

    @OptionLevel.setter
    def OptionLevel(self, value: int):
        ...

    @property
    def Type(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Brokerages.Tradier.TradierAccountType enum."""
        ...

    @Type.setter
    def Type(self, value: int):
        """This property contains the int value of a member of the QuantConnect.Brokerages.Tradier.TradierAccountType enum."""
        ...

    @property
    def LastUpdated(self) -> datetime.datetime:
        ...

    @LastUpdated.setter
    def LastUpdated(self, value: datetime.datetime):
        ...

    @property
    def Status(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Brokerages.Tradier.TradierAccountStatus enum."""
        ...

    @Status.setter
    def Status(self, value: int):
        """This property contains the int value of a member of the QuantConnect.Brokerages.Tradier.TradierAccountStatus enum."""
        ...

    @property
    def Classification(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Brokerages.Tradier.TradierAccountClassification enum."""
        ...

    @Classification.setter
    def Classification(self, value: int):
        """This property contains the int value of a member of the QuantConnect.Brokerages.Tradier.TradierAccountClassification enum."""
        ...

    def __init__(self) -> None:
        """Create a new account:"""
        ...


class TradierUser(System.Object):
    """User profile array:"""

    @property
    def Id(self) -> str:
        ...

    @Id.setter
    def Id(self, value: str):
        ...

    @property
    def Name(self) -> str:
        ...

    @Name.setter
    def Name(self, value: str):
        ...

    @property
    def Accounts(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierUserAccount]:
        ...

    @Accounts.setter
    def Accounts(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierUserAccount]):
        ...

    def __init__(self) -> None:
        ...


class TradierAccountType(System.Enum):
    """Tradier account type:"""

    DayTrader = 0

    Cash = 1

    Margin = 2


class TradierAccountTypeCash(System.Object):
    """Account Type Margin Settings:"""

    @property
    def Sweep(self) -> int:
        ...

    @Sweep.setter
    def Sweep(self, value: int):
        ...

    @property
    def CashAvailable(self) -> float:
        ...

    @CashAvailable.setter
    def CashAvailable(self, value: float):
        ...

    @property
    def UnsettledFunds(self) -> float:
        ...

    @UnsettledFunds.setter
    def UnsettledFunds(self, value: float):
        ...

    def __init__(self) -> None:
        ...


class TradierAccountTypeSettings(System.Object):
    """Common Account Settings."""

    @property
    def FedCall(self) -> float:
        ...

    @FedCall.setter
    def FedCall(self, value: float):
        ...

    @property
    def MaintenanceCall(self) -> float:
        ...

    @MaintenanceCall.setter
    def MaintenanceCall(self, value: float):
        ...

    @property
    def StockBuyingPower(self) -> float:
        ...

    @StockBuyingPower.setter
    def StockBuyingPower(self, value: float):
        ...

    @property
    def OptionBuyingPower(self) -> float:
        ...

    @OptionBuyingPower.setter
    def OptionBuyingPower(self, value: float):
        ...

    @property
    def StockShortValue(self) -> float:
        ...

    @StockShortValue.setter
    def StockShortValue(self, value: float):
        ...

    def __init__(self) -> None:
        ...


class TradierAccountTypeDayTrader(QuantConnect.Brokerages.Tradier.TradierAccountTypeSettings):
    """Account Type Day Trader Settings:"""

    @property
    def DayTradeBuyingPower(self) -> float:
        ...

    @DayTradeBuyingPower.setter
    def DayTradeBuyingPower(self, value: float):
        ...

    def __init__(self) -> None:
        ...


class TradierAccountTypeMargin(QuantConnect.Brokerages.Tradier.TradierAccountTypeSettings):
    """Account Type Margin Settings:"""

    @property
    def Sweep(self) -> int:
        ...

    @Sweep.setter
    def Sweep(self, value: int):
        ...

    def __init__(self) -> None:
        ...


class TradierBalanceDetails(System.Object):
    """Trader Balance Detail:"""

    @property
    def AccountNumber(self) -> str:
        ...

    @AccountNumber.setter
    def AccountNumber(self, value: str):
        ...

    @property
    def Type(self) -> QuantConnect.Brokerages.Tradier.TradierAccountType:
        ...

    @Type.setter
    def Type(self, value: QuantConnect.Brokerages.Tradier.TradierAccountType):
        ...

    @property
    def CashAvailable(self) -> float:
        ...

    @CashAvailable.setter
    def CashAvailable(self, value: float):
        ...

    @property
    def ClosingProfitLoss(self) -> float:
        ...

    @ClosingProfitLoss.setter
    def ClosingProfitLoss(self, value: float):
        ...

    @property
    def CurrentRequirement(self) -> float:
        ...

    @CurrentRequirement.setter
    def CurrentRequirement(self, value: float):
        ...

    @property
    def DividendBalance(self) -> float:
        ...

    @DividendBalance.setter
    def DividendBalance(self, value: float):
        ...

    @property
    def Equity(self) -> float:
        ...

    @Equity.setter
    def Equity(self, value: float):
        ...

    @property
    def LongLiquidValue(self) -> float:
        ...

    @LongLiquidValue.setter
    def LongLiquidValue(self, value: float):
        ...

    @property
    def LongMarketValue(self) -> float:
        ...

    @LongMarketValue.setter
    def LongMarketValue(self, value: float):
        ...

    @property
    def MarketValue(self) -> float:
        ...

    @MarketValue.setter
    def MarketValue(self, value: float):
        ...

    @property
    def NetValue(self) -> float:
        ...

    @NetValue.setter
    def NetValue(self, value: float):
        ...

    @property
    def OpenProfitLoss(self) -> float:
        ...

    @OpenProfitLoss.setter
    def OpenProfitLoss(self, value: float):
        ...

    @property
    def OptionLongValue(self) -> float:
        ...

    @OptionLongValue.setter
    def OptionLongValue(self, value: float):
        ...

    @property
    def OptionRequirement(self) -> float:
        ...

    @OptionRequirement.setter
    def OptionRequirement(self, value: float):
        ...

    @property
    def OptionShortValue(self) -> float:
        ...

    @OptionShortValue.setter
    def OptionShortValue(self, value: float):
        ...

    @property
    def PendingCash(self) -> float:
        ...

    @PendingCash.setter
    def PendingCash(self, value: float):
        ...

    @property
    def PendingOrdersCount(self) -> int:
        ...

    @PendingOrdersCount.setter
    def PendingOrdersCount(self, value: int):
        ...

    @property
    def ShortLiquidValue(self) -> float:
        ...

    @ShortLiquidValue.setter
    def ShortLiquidValue(self, value: float):
        ...

    @property
    def ShortMarketValue(self) -> float:
        ...

    @ShortMarketValue.setter
    def ShortMarketValue(self, value: float):
        ...

    @property
    def StockLongValue(self) -> float:
        ...

    @StockLongValue.setter
    def StockLongValue(self, value: float):
        ...

    @property
    def UnclearedFunds(self) -> float:
        ...

    @UnclearedFunds.setter
    def UnclearedFunds(self, value: float):
        ...

    @property
    def UnsettledFunds(self) -> float:
        ...

    @UnsettledFunds.setter
    def UnsettledFunds(self, value: float):
        ...

    @property
    def TotalCash(self) -> float:
        ...

    @TotalCash.setter
    def TotalCash(self, value: float):
        ...

    @property
    def TotalEquity(self) -> float:
        ...

    @TotalEquity.setter
    def TotalEquity(self, value: float):
        ...

    @property
    def CashTypeSettings(self) -> QuantConnect.Brokerages.Tradier.TradierAccountTypeCash:
        ...

    @CashTypeSettings.setter
    def CashTypeSettings(self, value: QuantConnect.Brokerages.Tradier.TradierAccountTypeCash):
        ...

    @property
    def PatternTraderTypeSettings(self) -> QuantConnect.Brokerages.Tradier.TradierAccountTypeDayTrader:
        ...

    @PatternTraderTypeSettings.setter
    def PatternTraderTypeSettings(self, value: QuantConnect.Brokerages.Tradier.TradierAccountTypeDayTrader):
        ...

    @property
    def MarginTypeSettings(self) -> QuantConnect.Brokerages.Tradier.TradierAccountTypeMargin:
        ...

    @MarginTypeSettings.setter
    def MarginTypeSettings(self, value: QuantConnect.Brokerages.Tradier.TradierAccountTypeMargin):
        ...


class TradierOrderType(System.Enum):
    """Tradier order type: (market, limit, stop, stop_limit or market) //credit, debit, even"""

    Limit = 0

    Market = 1

    StopLimit = 2

    StopMarket = 3

    Credit = 4

    Debit = 5

    Even = 6


class TradierOrderDirection(System.Enum):
    """
    Direction of the order
    (buy, buy_to_open, buy_to_cover, buy_to_close, sell, sell_short, sell_to_open, sell_to_close)
    """

    Buy = 0

    SellShort = 1

    Sell = 2

    BuyToCover = 3

    SellToOpen = 4

    SellToClose = 5

    BuyToClose = 6

    BuyToOpen = 7

    # Cannot convert to Python: None = 8


class TradierOrderStatus(System.Enum):
    """
    Status of the tradier order.
     (filled, canceled, open, expired, rejected, pending, partially_filled, submitted)
    """

    Filled = 0

    Canceled = 1

    Open = 2

    Expired = 3

    Rejected = 4

    Pending = 5

    PartiallyFilled = 6

    Submitted = 7


class TradierOrderDuration(System.Enum):
    """Length of the order offer."""

    GTC = 0

    Day = 1


class TradierOrderClass(System.Enum):
    """Class of the order."""

    Equity = 0

    Option = 1

    Multileg = 2

    Combo = 3


class TradierOrderLeg(System.Object):
    """Leg of a tradier order:"""

    @property
    def Type(self) -> QuantConnect.Brokerages.Tradier.TradierOrderType:
        ...

    @Type.setter
    def Type(self, value: QuantConnect.Brokerages.Tradier.TradierOrderType):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def Direction(self) -> QuantConnect.Brokerages.Tradier.TradierOrderDirection:
        ...

    @Direction.setter
    def Direction(self, value: QuantConnect.Brokerages.Tradier.TradierOrderDirection):
        ...

    @property
    def Quantity(self) -> float:
        ...

    @Quantity.setter
    def Quantity(self, value: float):
        ...

    @property
    def Status(self) -> QuantConnect.Brokerages.Tradier.TradierOrderStatus:
        ...

    @Status.setter
    def Status(self, value: QuantConnect.Brokerages.Tradier.TradierOrderStatus):
        ...

    @property
    def Duration(self) -> QuantConnect.Brokerages.Tradier.TradierOrderDuration:
        ...

    @Duration.setter
    def Duration(self, value: QuantConnect.Brokerages.Tradier.TradierOrderDuration):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def AverageFillPrice(self) -> float:
        ...

    @AverageFillPrice.setter
    def AverageFillPrice(self, value: float):
        ...

    @property
    def QuantityExecuted(self) -> float:
        ...

    @QuantityExecuted.setter
    def QuantityExecuted(self, value: float):
        ...

    @property
    def LastFillPrice(self) -> float:
        ...

    @LastFillPrice.setter
    def LastFillPrice(self, value: float):
        ...

    @property
    def LastFillQuantity(self) -> float:
        ...

    @LastFillQuantity.setter
    def LastFillQuantity(self, value: float):
        ...

    @property
    def RemainingQuantity(self) -> float:
        ...

    @RemainingQuantity.setter
    def RemainingQuantity(self, value: float):
        ...

    @property
    def CreatedDate(self) -> datetime.datetime:
        ...

    @CreatedDate.setter
    def CreatedDate(self, value: datetime.datetime):
        ...

    @property
    def TransacionDate(self) -> datetime.datetime:
        ...

    @TransacionDate.setter
    def TransacionDate(self, value: datetime.datetime):
        ...

    def __init__(self) -> None:
        ...


class TradierOrder(System.Object):
    """Intraday or pending order for user"""

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def Type(self) -> QuantConnect.Brokerages.Tradier.TradierOrderType:
        ...

    @Type.setter
    def Type(self, value: QuantConnect.Brokerages.Tradier.TradierOrderType):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def OptionSymbol(self) -> str:
        ...

    @OptionSymbol.setter
    def OptionSymbol(self, value: str):
        ...

    @property
    def Direction(self) -> QuantConnect.Brokerages.Tradier.TradierOrderDirection:
        ...

    @Direction.setter
    def Direction(self, value: QuantConnect.Brokerages.Tradier.TradierOrderDirection):
        ...

    @property
    def Quantity(self) -> float:
        ...

    @Quantity.setter
    def Quantity(self, value: float):
        ...

    @property
    def Status(self) -> QuantConnect.Brokerages.Tradier.TradierOrderStatus:
        ...

    @Status.setter
    def Status(self, value: QuantConnect.Brokerages.Tradier.TradierOrderStatus):
        ...

    @property
    def Duration(self) -> QuantConnect.Brokerages.Tradier.TradierOrderDuration:
        ...

    @Duration.setter
    def Duration(self, value: QuantConnect.Brokerages.Tradier.TradierOrderDuration):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def AverageFillPrice(self) -> float:
        ...

    @AverageFillPrice.setter
    def AverageFillPrice(self, value: float):
        ...

    @property
    def QuantityExecuted(self) -> float:
        ...

    @QuantityExecuted.setter
    def QuantityExecuted(self, value: float):
        ...

    @property
    def LastFillPrice(self) -> float:
        ...

    @LastFillPrice.setter
    def LastFillPrice(self, value: float):
        ...

    @property
    def LastFillQuantity(self) -> float:
        ...

    @LastFillQuantity.setter
    def LastFillQuantity(self, value: float):
        ...

    @property
    def RemainingQuantity(self) -> float:
        ...

    @RemainingQuantity.setter
    def RemainingQuantity(self, value: float):
        ...

    @property
    def CreatedDate(self) -> datetime.datetime:
        ...

    @CreatedDate.setter
    def CreatedDate(self, value: datetime.datetime):
        ...

    @property
    def TransactionDate(self) -> datetime.datetime:
        ...

    @TransactionDate.setter
    def TransactionDate(self, value: datetime.datetime):
        ...

    @property
    def Class(self) -> QuantConnect.Brokerages.Tradier.TradierOrderClass:
        ...

    @Class.setter
    def Class(self, value: QuantConnect.Brokerages.Tradier.TradierOrderClass):
        ...

    @property
    def NumberOfLegs(self) -> int:
        ...

    @NumberOfLegs.setter
    def NumberOfLegs(self, value: int):
        ...

    @property
    def Legs(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierOrderLeg]:
        ...

    @Legs.setter
    def Legs(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierOrderLeg]):
        ...

    def __init__(self) -> None:
        ...


class TradierOptionType(System.Enum):
    """Tradier option type"""

    Put = 0

    Call = 1


class TradierOrderDetailed(QuantConnect.Brokerages.Tradier.TradierOrder):
    """Detailed order type."""

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def ExecutionExchange(self) -> str:
        ...

    @ExecutionExchange.setter
    def ExecutionExchange(self, value: str):
        ...

    @property
    def OptionType(self) -> QuantConnect.Brokerages.Tradier.TradierOptionType:
        ...

    @OptionType.setter
    def OptionType(self, value: QuantConnect.Brokerages.Tradier.TradierOptionType):
        ...

    @property
    def OptionExpirationDate(self) -> datetime.datetime:
        ...

    @OptionExpirationDate.setter
    def OptionExpirationDate(self, value: datetime.datetime):
        ...

    @property
    def StopPrice(self) -> float:
        ...

    @StopPrice.setter
    def StopPrice(self, value: float):
        ...


class TradierOrderResponseOrder(System.Object):
    """Order response when purchasing equity."""

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def PartnerId(self) -> str:
        ...

    @PartnerId.setter
    def PartnerId(self, value: str):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...


class TradierOrderResponseError(System.Object):
    """Errors result from an order request."""

    @property
    def Errors(self) -> System.Collections.Generic.List[str]:
        ...

    @Errors.setter
    def Errors(self, value: System.Collections.Generic.List[str]):
        ...


class TradierOrderResponse(System.Object):
    """Deserialization wrapper for order response:"""

    @property
    def Order(self) -> QuantConnect.Brokerages.Tradier.TradierOrderResponseOrder:
        ...

    @Order.setter
    def Order(self, value: QuantConnect.Brokerages.Tradier.TradierOrderResponseOrder):
        ...

    @property
    def Errors(self) -> QuantConnect.Brokerages.Tradier.TradierOrderResponseError:
        ...

    @Errors.setter
    def Errors(self, value: QuantConnect.Brokerages.Tradier.TradierOrderResponseError):
        ...


class TradierTimeSeriesIntervals(System.Enum):
    """TradeBar windows for Tradier's data histories"""

    Tick = 0

    OneMinute = 1

    FiveMinutes = 2

    FifteenMinutes = 3


class TradierHistoricalDataIntervals(System.Enum):
    """Historical data intervals for tradier requests:"""

    Daily = 0

    Weekly = 1

    Monthly = 2


class TradierMarketStatus(System.Object):
    """Current market status description"""

    @property
    def Date(self) -> datetime.datetime:
        ...

    @Date.setter
    def Date(self, value: datetime.datetime):
        ...

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...

    @property
    def NextChange(self) -> str:
        ...

    @NextChange.setter
    def NextChange(self, value: str):
        ...

    @property
    def State(self) -> str:
        ...

    @State.setter
    def State(self, value: str):
        ...

    @property
    def TimeStamp(self) -> int:
        ...

    @TimeStamp.setter
    def TimeStamp(self, value: int):
        ...


class TradierPosition(System.Object):
    """Individual Tradier position model."""

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def DateAcquired(self) -> datetime.datetime:
        ...

    @DateAcquired.setter
    def DateAcquired(self, value: datetime.datetime):
        ...

    @property
    def Quantity(self) -> int:
        ...

    @Quantity.setter
    def Quantity(self, value: int):
        ...

    @property
    def CostBasis(self) -> float:
        ...

    @CostBasis.setter
    def CostBasis(self, value: float):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...


class TradierEventType(System.Enum):
    """Tradier event type:"""

    Trade = 0

    Journal = 1

    Option = 2

    Dividend = 3


class TradierEventDetail(System.Object):
    """Common base class for events detail information:"""

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...

    @property
    def Quantity(self) -> float:
        ...

    @Quantity.setter
    def Quantity(self, value: float):
        ...

    def __init__(self) -> None:
        ...


class TradierTradeType(System.Enum):
    """Market type of the trade:"""

    Equity = 0

    Option = 1


class TradierTradeEvent(QuantConnect.Brokerages.Tradier.TradierEventDetail):
    """Trade event in history for tradier:"""

    @property
    def Commission(self) -> float:
        ...

    @Commission.setter
    def Commission(self, value: float):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def TradeType(self) -> QuantConnect.Brokerages.Tradier.TradierTradeType:
        ...

    @TradeType.setter
    def TradeType(self, value: QuantConnect.Brokerages.Tradier.TradierTradeType):
        ...

    def __init__(self) -> None:
        ...


class TradierJournalEvent(QuantConnect.Brokerages.Tradier.TradierEventDetail):
    """Journal event in history:"""

    def __init__(self) -> None:
        ...


class TradierOptionStatus(System.Enum):
    """Tradier options status"""

    Exercise = 0

    Expired = 1

    Assignment = 2


class TradierOptionEvent(QuantConnect.Brokerages.Tradier.TradierEventDetail):
    """Option event record in history:"""

    @property
    def Type(self) -> QuantConnect.Brokerages.Tradier.TradierOptionStatus:
        ...

    @Type.setter
    def Type(self, value: QuantConnect.Brokerages.Tradier.TradierOptionStatus):
        ...

    def __init__(self) -> None:
        ...


class TradierEvent(System.Object):
    """Tradier event model:"""

    @property
    def Amount(self) -> float:
        ...

    @Amount.setter
    def Amount(self, value: float):
        ...

    @property
    def Date(self) -> datetime.datetime:
        ...

    @Date.setter
    def Date(self, value: datetime.datetime):
        ...

    @property
    def Type(self) -> QuantConnect.Brokerages.Tradier.TradierEventType:
        ...

    @Type.setter
    def Type(self, value: QuantConnect.Brokerages.Tradier.TradierEventType):
        ...

    @property
    def TradeEvent(self) -> QuantConnect.Brokerages.Tradier.TradierTradeEvent:
        ...

    @TradeEvent.setter
    def TradeEvent(self, value: QuantConnect.Brokerages.Tradier.TradierTradeEvent):
        ...

    @property
    def JournalEvent(self) -> QuantConnect.Brokerages.Tradier.TradierJournalEvent:
        ...

    @JournalEvent.setter
    def JournalEvent(self, value: QuantConnect.Brokerages.Tradier.TradierJournalEvent):
        ...

    @property
    def OptionEvent(self) -> QuantConnect.Brokerages.Tradier.TradierOptionEvent:
        ...

    @OptionEvent.setter
    def OptionEvent(self, value: QuantConnect.Brokerages.Tradier.TradierOptionEvent):
        ...

    @property
    def DividendEvent(self) -> QuantConnect.Brokerages.Tradier.TradierOptionEvent:
        ...

    @DividendEvent.setter
    def DividendEvent(self, value: QuantConnect.Brokerages.Tradier.TradierOptionEvent):
        ...


class TradierGainLoss(System.Object):
    """Account only settings for a tradier user:"""

    @property
    def CloseDate(self) -> datetime.datetime:
        ...

    @CloseDate.setter
    def CloseDate(self, value: datetime.datetime):
        ...

    @property
    def OpenDate(self) -> datetime.datetime:
        ...

    @OpenDate.setter
    def OpenDate(self, value: datetime.datetime):
        ...

    @property
    def Cost(self) -> float:
        ...

    @Cost.setter
    def Cost(self, value: float):
        ...

    @property
    def GainLoss(self) -> float:
        ...

    @GainLoss.setter
    def GainLoss(self, value: float):
        ...

    @property
    def GainLossPercentage(self) -> float:
        ...

    @GainLossPercentage.setter
    def GainLossPercentage(self, value: float):
        ...

    @property
    def Proceeds(self) -> float:
        ...

    @Proceeds.setter
    def Proceeds(self, value: float):
        ...

    @property
    def Quantity(self) -> float:
        ...

    @Quantity.setter
    def Quantity(self, value: float):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def Term(self) -> float:
        ...

    @Term.setter
    def Term(self, value: float):
        ...

    def __init__(self) -> None:
        """Closed position trade summary"""
        ...


class TradierQuote(System.Object):
    """Quote data from Tradier:"""

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def Last(self) -> float:
        ...

    @Last.setter
    def Last(self, value: float):
        ...

    @property
    def Change(self) -> float:
        ...

    @Change.setter
    def Change(self, value: float):
        ...

    @property
    def PercentageChange(self) -> float:
        ...

    @PercentageChange.setter
    def PercentageChange(self, value: float):
        ...

    @property
    def Volume(self) -> float:
        ...

    @Volume.setter
    def Volume(self, value: float):
        ...

    @property
    def AverageVolume(self) -> float:
        ...

    @AverageVolume.setter
    def AverageVolume(self, value: float):
        ...

    @property
    def LastVolume(self) -> float:
        ...

    @LastVolume.setter
    def LastVolume(self, value: float):
        ...

    @property
    def TradeDateUnix(self) -> int:
        ...

    @TradeDateUnix.setter
    def TradeDateUnix(self, value: int):
        ...

    @property
    def Open(self) -> typing.Optional[float]:
        ...

    @Open.setter
    def Open(self, value: typing.Optional[float]):
        ...

    @property
    def High(self) -> typing.Optional[float]:
        ...

    @High.setter
    def High(self, value: typing.Optional[float]):
        ...

    @property
    def Low(self) -> typing.Optional[float]:
        ...

    @Low.setter
    def Low(self, value: typing.Optional[float]):
        ...

    @property
    def Close(self) -> typing.Optional[float]:
        ...

    @Close.setter
    def Close(self, value: typing.Optional[float]):
        ...

    @property
    def PreviousClose(self) -> float:
        ...

    @PreviousClose.setter
    def PreviousClose(self, value: float):
        ...

    @property
    def Week52High(self) -> float:
        ...

    @Week52High.setter
    def Week52High(self, value: float):
        ...

    @property
    def Week52Low(self) -> float:
        ...

    @Week52Low.setter
    def Week52Low(self, value: float):
        ...

    @property
    def Bid(self) -> typing.Optional[float]:
        ...

    @Bid.setter
    def Bid(self, value: typing.Optional[float]):
        ...

    @property
    def BidSize(self) -> float:
        ...

    @BidSize.setter
    def BidSize(self, value: float):
        ...

    @property
    def BigExchange(self) -> str:
        ...

    @BigExchange.setter
    def BigExchange(self, value: str):
        ...

    @property
    def Ask(self) -> typing.Optional[float]:
        ...

    @Ask.setter
    def Ask(self, value: typing.Optional[float]):
        ...

    @property
    def AskSize(self) -> float:
        ...

    @AskSize.setter
    def AskSize(self, value: float):
        ...

    @property
    def AskExchange(self) -> str:
        ...

    @AskExchange.setter
    def AskExchange(self, value: str):
        ...

    def __init__(self) -> None:
        ...


class TradierTimeSeries(System.Object):
    """One bar of historical Tradier data."""

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Open(self) -> float:
        ...

    @Open.setter
    def Open(self, value: float):
        ...

    @property
    def High(self) -> float:
        ...

    @High.setter
    def High(self, value: float):
        ...

    @property
    def Low(self) -> float:
        ...

    @Low.setter
    def Low(self, value: float):
        ...

    @property
    def Close(self) -> float:
        ...

    @Close.setter
    def Close(self, value: float):
        ...

    @property
    def Volume(self) -> int:
        ...

    @Volume.setter
    def Volume(self, value: int):
        ...


class TradierHistoryBar(System.Object):
    """"Bar" for a history unit."""

    @property
    def Time(self) -> datetime.datetime:
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        ...

    @property
    def Open(self) -> float:
        ...

    @Open.setter
    def Open(self, value: float):
        ...

    @property
    def High(self) -> float:
        ...

    @High.setter
    def High(self, value: float):
        ...

    @property
    def Low(self) -> float:
        ...

    @Low.setter
    def Low(self, value: float):
        ...

    @property
    def Close(self) -> float:
        ...

    @Close.setter
    def Close(self, value: float):
        ...

    @property
    def Volume(self) -> int:
        ...

    @Volume.setter
    def Volume(self, value: int):
        ...


class TradierCalendarDayMarketHours(System.Object):
    """Start and finish time of market hours for this market."""

    @property
    def Start(self) -> datetime.datetime:
        ...

    @Start.setter
    def Start(self, value: datetime.datetime):
        ...

    @property
    def End(self) -> datetime.datetime:
        ...

    @End.setter
    def End(self, value: datetime.datetime):
        ...


class TradierCalendarDay(System.Object):
    """Single days properties from the calendar:"""

    @property
    def Date(self) -> datetime.datetime:
        ...

    @Date.setter
    def Date(self, value: datetime.datetime):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...

    @property
    def Premarket(self) -> QuantConnect.Brokerages.Tradier.TradierCalendarDayMarketHours:
        ...

    @Premarket.setter
    def Premarket(self, value: QuantConnect.Brokerages.Tradier.TradierCalendarDayMarketHours):
        ...

    @property
    def Open(self) -> QuantConnect.Brokerages.Tradier.TradierCalendarDayMarketHours:
        ...

    @Open.setter
    def Open(self, value: QuantConnect.Brokerages.Tradier.TradierCalendarDayMarketHours):
        ...

    @property
    def Postmarket(self) -> QuantConnect.Brokerages.Tradier.TradierCalendarDayMarketHours:
        ...

    @Postmarket.setter
    def Postmarket(self, value: QuantConnect.Brokerages.Tradier.TradierCalendarDayMarketHours):
        ...


class TradierSearchResult(System.Object):
    """One search result from API"""

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...


class TradierBrokerage(QuantConnect.Brokerages.BaseWebsocketsBrokerage, QuantConnect.Interfaces.IDataQueueHandler, QuantConnect.Interfaces.IDataQueueUniverseProvider, QuantConnect.Interfaces.IHistoryProvider):
    """
    Tradier Class:
     - Handle authentication.
     - Data requests.
     - Rate limiting.
     - Placing orders.
     - Getting user data.
    """

    @property
    def AccountBaseCurrency(self) -> str:
        """Returns the brokerage account's base currency"""
        ...

    @AccountBaseCurrency.setter
    def AccountBaseCurrency(self, value: str):
        """Returns the brokerage account's base currency"""
        ...

    @property
    def IsConnected(self) -> bool:
        ...

    @property
    def InvalidConfigurationDetected(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.InvalidConfigurationDetectedEventArgs], None], None]:
        ...

    @InvalidConfigurationDetected.setter
    def InvalidConfigurationDetected(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.InvalidConfigurationDetectedEventArgs], None], None]):
        ...

    @property
    def NumericalPrecisionLimited(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.NumericalPrecisionLimitedEventArgs], None], None]:
        """Event fired when the numerical precision in the factor file has been limited"""
        ...

    @NumericalPrecisionLimited.setter
    def NumericalPrecisionLimited(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.NumericalPrecisionLimitedEventArgs], None], None]):
        """Event fired when the numerical precision in the factor file has been limited"""
        ...

    @property
    def DownloadFailed(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.DownloadFailedEventArgs], None], None]:
        """Event fired when there was an error downloading a remote file"""
        ...

    @DownloadFailed.setter
    def DownloadFailed(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.DownloadFailedEventArgs], None], None]):
        """Event fired when there was an error downloading a remote file"""
        ...

    @property
    def ReaderErrorDetected(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.ReaderErrorDetectedEventArgs], None], None]:
        """Event fired when there was an error reading the data"""
        ...

    @ReaderErrorDetected.setter
    def ReaderErrorDetected(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.ReaderErrorDetectedEventArgs], None], None]):
        """Event fired when there was an error reading the data"""
        ...

    @property
    def StartDateLimited(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.StartDateLimitedEventArgs], None], None]:
        """Event fired when the start date has been limited"""
        ...

    @StartDateLimited.setter
    def StartDateLimited(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.StartDateLimitedEventArgs], None], None]):
        """Event fired when the start date has been limited"""
        ...

    @property
    def DataPointCount(self) -> int:
        ...

    @DataPointCount.setter
    def DataPointCount(self, value: int):
        ...

    @typing.overload
    def __init__(self) -> None:
        """Create a new Tradier Object:"""
        ...

    @typing.overload
    def __init__(self, algorithm: QuantConnect.Interfaces.IAlgorithm, orderProvider: QuantConnect.Securities.IOrderProvider, securityProvider: QuantConnect.Securities.ISecurityProvider, aggregator: QuantConnect.Data.IDataAggregator, useSandbox: bool, accountId: str, accessToken: str) -> None:
        """Create a new Tradier Object:"""
        ...

    @typing.overload
    def CancelOrder(self, orderId: int) -> QuantConnect.Brokerages.Tradier.TradierOrderResponse:
        """Cancel the order with this account and id number"""
        ...

    @typing.overload
    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Cancels the order with the specified ID
        
        :param order: The order to cancel
        :returns: True if the request was made for the order to be canceled, false otherwise.
        """
        ...

    def CanPerformSelection(self) -> bool:
        """
        Returns whether selection can take place or not.
        
        :returns: True if selection can take place.
        """
        ...

    def ChangeOrder(self, orderId: int, type: QuantConnect.Brokerages.Tradier.TradierOrderType = ..., duration: QuantConnect.Brokerages.Tradier.TradierOrderDuration = ..., price: float = 0, stop: float = 0) -> QuantConnect.Brokerages.Tradier.TradierOrderResponse:
        """Update an exiting Tradier Order:"""
        ...

    @staticmethod
    def ConvertDirection(direction: QuantConnect.Orders.OrderDirection, securityType: QuantConnect.SecurityType, holdingQuantity: float) -> int:
        """
        Converts the QC order direction to a tradier order direction
        
        This method is protected.
        
        :returns: This method returns the int value of a member of the QuantConnect.Brokerages.Tradier.TradierOrderDirection enum.
        """
        ...

    def ConvertHolding(self, position: QuantConnect.Brokerages.Tradier.TradierPosition) -> QuantConnect.Holding:
        """
        Converts the tradier position into a qc holding
        
        This method is protected.
        """
        ...

    def ConvertOrder(self, order: QuantConnect.Brokerages.Tradier.TradierOrder) -> QuantConnect.Orders.Order:
        """
        Converts the specified tradier order into a qc order.
        The 'task' will have a value if we needed to issue a rest call for the stop price, otherwise it will be null
        
        This method is protected.
        """
        ...

    @typing.overload
    def ConvertOrderType(self, type: QuantConnect.Orders.OrderType) -> int:
        """
        Converts the qc order type into a tradier order type
        
        This method is protected.
        
        :returns: This method returns the int value of a member of the QuantConnect.Brokerages.Tradier.TradierOrderType enum.
        """
        ...

    @staticmethod
    @typing.overload
    def ConvertOrderType(order: QuantConnect.Orders.Order) -> int:
        """
        Converts the qc order type into a tradier order type
        
        This method is protected.
        
        :returns: This method returns the int value of a member of the QuantConnect.Brokerages.Tradier.TradierOrderType enum.
        """
        ...

    def ConvertQuantity(self, order: QuantConnect.Brokerages.Tradier.TradierOrder) -> int:
        """
        Converts the tradier order quantity into a qc quantity
        
        This method is protected.
        """
        ...

    @typing.overload
    def ConvertStatus(self, status: QuantConnect.Brokerages.Tradier.TradierOrderStatus) -> int:
        """
        Converts the tradier order status into a qc order status
        
        This method is protected.
        
        :returns: This method returns the int value of a member of the QuantConnect.Orders.OrderStatus enum.
        """
        ...

    @typing.overload
    def ConvertStatus(self, status: QuantConnect.Orders.OrderStatus) -> int:
        """
        Converts the qc order status into a tradier order status
        
        This method is protected.
        
        :returns: This method returns the int value of a member of the QuantConnect.Brokerages.Tradier.TradierOrderStatus enum.
        """
        ...

    def ConvertTimeInForce(self, duration: QuantConnect.Brokerages.Tradier.TradierOrderDuration) -> QuantConnect.Orders.TimeInForce:
        """
        Converts the tradier order duration into a qc order time in force
        
        This method is protected.
        """
        ...

    def Disconnect(self) -> None:
        """Disconnects the client from the broker's remote servers"""
        ...

    def Dispose(self) -> None:
        """Dispose of the brokerage instance"""
        ...

    def GetAccountEvents(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierEvent]:
        """Get a list of historical events for this account:"""
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """
        Gets all holdings for the account
        
        :returns: The current holdings from the account.
        """
        ...

    def GetBalanceDetails(self) -> QuantConnect.Brokerages.Tradier.TradierBalanceDetails:
        """
        Get all the users balance information:
        
        :returns: Balance.
        """
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """
        Gets the current cash balance for each currency held in the brokerage account
        
        :returns: The current cash balance for each currency available for trading.
        """
        ...

    def GetGainLoss(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierGainLoss]:
        """GainLoss of recent trades for this account:"""
        ...

    def GetHistoricalData(self, symbol: typing.Union[QuantConnect.Symbol, str], start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date], interval: QuantConnect.Brokerages.Tradier.TradierHistoricalDataIntervals = ...) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierHistoryBar]:
        """Get full daily, weekly or monthly bars of historical periods:"""
        ...

    def GetHistory(self, requests: System.Collections.Generic.IEnumerable[QuantConnect.Data.HistoryRequest], sliceTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Slice]:
        """
        Gets the history for the requested securities
        
        :param requests: The historical data requests
        :param sliceTimeZone: The time zone used when time stamping the slice instances
        :returns: An enumerable of the slices of data covering the span specified in each request.
        """
        ...

    @staticmethod
    def GetLimitPrice(order: QuantConnect.Orders.Order) -> float:
        """
        Gets the limit price used in API calls with tradier from the specified qc order instance
        
        This method is protected.
        """
        ...

    def GetMarketCalendar(self, month: int, year: int) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierCalendarDay]:
        """Get the list of days status for this calendar month, year:"""
        ...

    def GetMarketStatus(self) -> QuantConnect.Brokerages.Tradier.TradierMarketStatus:
        """Get the current market status"""
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """
        Gets all open orders on the account.
        NOTE: The order objects returned do not have QC order IDs.
        
        :returns: The open orders returned from IB.
        """
        ...

    def GetOrder(self, orderId: int) -> QuantConnect.Brokerages.Tradier.TradierOrderDetailed:
        """Get information about a specific order: accounts/{account_id}/orders/{id}"""
        ...

    @staticmethod
    def GetOrderDuration(timeInForce: QuantConnect.Orders.TimeInForce) -> int:
        """
        Converts the qc order duration into a tradier order duration
        
        This method is protected.
        
        :returns: This method returns the int value of a member of the QuantConnect.Brokerages.Tradier.TradierOrderDuration enum.
        """
        ...

    def GetPositions(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierPosition]:
        """
        Get a list of the tradier positions for this account:
        
        :returns: Array of the symbols we hold.
        """
        ...

    def GetQuotes(self, symbols: System.Collections.Generic.List[str]) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierQuote]:
        """List of quotes for symbols"""
        ...

    @staticmethod
    def GetStopPrice(order: QuantConnect.Orders.Order) -> float:
        """
        Gets the stop price used in API calls with tradier from the specified qc order instance
        
        This method is protected.
        """
        ...

    def GetTimeSeries(self, symbol: typing.Union[QuantConnect.Symbol, str], start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date], interval: QuantConnect.Brokerages.Tradier.TradierTimeSeriesIntervals) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierTimeSeries]:
        """Get the historical bars for this period"""
        ...

    def GetUserProfile(self) -> QuantConnect.Brokerages.Tradier.TradierUser:
        """
        Using this auth token get the tradier user:
        
        :returns: Tradier user model:.
        """
        ...

    def Initialize(self, parameters: QuantConnect.Data.HistoryProviderInitializeParameters) -> None:
        """
        Initializes this history provider to work for the specified job
        
        :param parameters: The initialization parameters
        """
        ...

    @staticmethod
    def IsShort(direction: QuantConnect.Brokerages.Tradier.TradierOrderDirection) -> bool:
        """
        Returns true if the specified tradier order direction represents a short position
        
        This method is protected.
        """
        ...

    def LookUpSymbol(self, query: str, includeIndexes: bool = True) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierSearchResult]:
        """Get the list of days status for this calendar month, year:"""
        ...

    def LookupSymbols(self, symbol: typing.Union[QuantConnect.Symbol, str], includeExpired: bool, securityCurrency: str = None) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        """
        Method returns a collection of Symbols that are available at the data source.
        
        :param symbol: Symbol to lookup
        :param includeExpired: Include expired contracts
        :param securityCurrency: Expected security currency(if any)
        :returns: Enumerable of Symbols, that are associated with the provided Symbol.
        """
        ...

    def OnDownloadFailed(self, e: QuantConnect.DownloadFailedEventArgs) -> None:
        """
        Event invocator for the DownloadFailed event
        
        This method is protected.
        
        :param e: Event arguments for the DownloadFailed event
        """
        ...

    def OnInvalidConfigurationDetected(self, e: QuantConnect.InvalidConfigurationDetectedEventArgs) -> None:
        """
        Event invocator for the InvalidConfigurationDetected event
        
        This method is protected.
        
        :param e: Event arguments for the InvalidConfigurationDetected event
        """
        ...

    @typing.overload
    def OnMessage(self, e: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        """
        Event invocator for the Message event
        
        This method is protected.
        
        :param e: The error
        """
        ...

    @typing.overload
    def OnMessage(self, sender: typing.Any, webSocketMessage: QuantConnect.Brokerages.WebSocketMessage) -> None:
        """
        Handles websocket received messages
        
        This method is protected.
        """
        ...

    def OnNumericalPrecisionLimited(self, e: QuantConnect.NumericalPrecisionLimitedEventArgs) -> None:
        """
        Event invocator for the NumericalPrecisionLimited event
        
        This method is protected.
        
        :param e: Event arguments for the NumericalPrecisionLimited event
        """
        ...

    def OnReaderErrorDetected(self, e: QuantConnect.ReaderErrorDetectedEventArgs) -> None:
        """
        Event invocator for the ReaderErrorDetected event
        
        This method is protected.
        
        :param e: Event arguments for the ReaderErrorDetected event
        """
        ...

    def OrderCrossesZero(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Determines whether or not the specified order will bring us across the zero line for holdings
        
        This method is protected.
        """
        ...

    @staticmethod
    def OrderIsClosed(order: QuantConnect.Brokerages.Tradier.TradierOrder) -> bool:
        """
        Returns true if the specified order is considered close, otherwise false
        
        This method is protected.
        """
        ...

    @staticmethod
    def OrderIsOpen(order: QuantConnect.Brokerages.Tradier.TradierOrder) -> bool:
        """This method is protected."""
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Places a new order and assigns a new broker ID to the order
        
        :param order: The order to be placed
        :returns: True if the request for a new order has been placed, false otherwise.
        """
        ...

    def Search(self, query: str, includeIndexes: bool = True) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierSearchResult]:
        """Get the list of days status for this calendar month, year:"""
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Sets the job we're subscribing for
        
        :param job: Job we're subscribing for
        """
        ...

    @typing.overload
    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: typing.Callable[[System.Object, System.EventArgs], None]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    @typing.overload
    def Subscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> bool:
        """This method is protected."""
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Updates the order with the same id
        
        :param order: The new order information
        :returns: True if the request was made for the order to be updated, false otherwise.
        """
        ...


class TradierSymbolMapper(System.Object, QuantConnect.Brokerages.ISymbolMapper):
    """Provides the mapping between Lean symbols and Tradier symbols."""

    def GetBrokerageSymbol(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> str:
        """
        Converts a Lean symbol instance to a Tradier symbol
        
        :param symbol: A Lean symbol instance
        :returns: The Tradier symbol.
        """
        ...

    @typing.overload
    def GetLeanSymbol(self, brokerageSymbol: str, securityType: QuantConnect.SecurityType, market: str, expirationDate: typing.Union[datetime.datetime, datetime.date] = ..., strike: float = 0, optionRight: QuantConnect.OptionRight = ...) -> QuantConnect.Symbol:
        """
        Converts a Tradier symbol to a Lean symbol instance
        
        :param brokerageSymbol: The Tradier symbol
        :param securityType: The security type
        :param market: The market
        :param expirationDate: Expiration date of the security(if applicable)
        :param strike: The strike of the security (if applicable)
        :param optionRight: The option right of the security (if applicable)
        :returns: A new Lean Symbol instance.
        """
        ...

    @typing.overload
    def GetLeanSymbol(self, brokerageSymbol: str) -> QuantConnect.Symbol:
        """
        Converts a Tradier symbol to a Lean symbol instance
        
        :param brokerageSymbol: The Tradier symbol
        :returns: A new Lean Symbol instance.
        """
        ...


class TradierApiRequestType(System.Enum):
    """Rate limiting categorization"""

    Standard = 0

    Data = 1

    Orders = 2


class TradierAccountStatus(System.Enum):
    """Account status flag."""

    New = 0

    Approved = 1

    Closed = 2


class TradierOptionExpirationType(System.Enum):
    """Tradier options expiration"""

    Standard = 0

    Weekly = 1


class TradierAccountClassification(System.Enum):
    """Account classification"""

    Individual = 0

    IRA = 1

    Roth_Ira = 2

    Joint = 3

    Entity = 4


class TradierTimeSeriesContainer(System.Object):
    """Container for timeseries array"""

    @property
    def TimeSeries(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierTimeSeries]:
        ...

    @TimeSeries.setter
    def TimeSeries(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierTimeSeries]):
        ...


class TradierQuoteContainer(System.Object):
    """Container for quotes:"""

    @property
    def Quotes(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierQuote]:
        ...

    @Quotes.setter
    def Quotes(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierQuote]):
        ...


class TradierHistoryDataContainer(System.Object):
    """Container for deserializing history classes"""

    @property
    def Data(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierHistoryBar]:
        ...

    @Data.setter
    def Data(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierHistoryBar]):
        ...


class TradierCalendarDayContainer(System.Object):
    """Container for the days array:"""

    @property
    def Days(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierCalendarDay]:
        ...

    @Days.setter
    def Days(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierCalendarDay]):
        ...


class TradierCalendarStatus(System.Object):
    """Calendar status:"""

    @property
    def Days(self) -> QuantConnect.Brokerages.Tradier.TradierCalendarDayContainer:
        ...

    @Days.setter
    def Days(self, value: QuantConnect.Brokerages.Tradier.TradierCalendarDayContainer):
        ...

    @property
    def Month(self) -> int:
        ...

    @Month.setter
    def Month(self, value: int):
        ...

    @property
    def Year(self) -> int:
        ...

    @Year.setter
    def Year(self, value: int):
        ...


class TradierSearchContainer(System.Object):
    """Tradier Search Container for Deserialization:"""

    @property
    def Results(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierSearchResult]:
        ...

    @Results.setter
    def Results(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierSearchResult]):
        ...


class TradierStreamSession(System.Object):
    """Create a new stream session"""

    @property
    def SessionId(self) -> str:
        ...

    @SessionId.setter
    def SessionId(self, value: str):
        ...

    @property
    def Url(self) -> str:
        ...

    @Url.setter
    def Url(self, value: str):
        ...


class TradierStreamData(System.Object):
    """One data packet from a tradier stream:"""

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def UnixDate(self) -> str:
        ...

    @UnixDate.setter
    def UnixDate(self, value: str):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def SummaryOpen(self) -> float:
        ...

    @SummaryOpen.setter
    def SummaryOpen(self, value: float):
        ...

    @property
    def SummaryHigh(self) -> float:
        ...

    @SummaryHigh.setter
    def SummaryHigh(self, value: float):
        ...

    @property
    def SummaryLow(self) -> float:
        ...

    @SummaryLow.setter
    def SummaryLow(self, value: float):
        ...

    @property
    def SummaryClose(self) -> float:
        ...

    @SummaryClose.setter
    def SummaryClose(self, value: float):
        ...

    @property
    def BidPrice(self) -> float:
        ...

    @BidPrice.setter
    def BidPrice(self, value: float):
        ...

    @property
    def BidSize(self) -> int:
        ...

    @BidSize.setter
    def BidSize(self, value: int):
        ...

    @property
    def BidExchange(self) -> str:
        ...

    @BidExchange.setter
    def BidExchange(self, value: str):
        ...

    @property
    def BidDateUnix(self) -> int:
        ...

    @BidDateUnix.setter
    def BidDateUnix(self, value: int):
        ...

    @property
    def TradePrice(self) -> float:
        ...

    @TradePrice.setter
    def TradePrice(self, value: float):
        ...

    @property
    def TradeSize(self) -> float:
        ...

    @TradeSize.setter
    def TradeSize(self, value: float):
        ...

    @property
    def TradeExchange(self) -> str:
        ...

    @TradeExchange.setter
    def TradeExchange(self, value: str):
        ...

    @property
    def TradeCVol(self) -> int:
        ...

    @TradeCVol.setter
    def TradeCVol(self, value: int):
        ...

    @property
    def AskPrice(self) -> float:
        ...

    @AskPrice.setter
    def AskPrice(self, value: float):
        ...

    @property
    def AskSize(self) -> int:
        ...

    @AskSize.setter
    def AskSize(self, value: int):
        ...

    @property
    def AskExchange(self) -> str:
        ...

    @AskExchange.setter
    def AskExchange(self, value: str):
        ...

    @property
    def AskDateUnix(self) -> int:
        ...

    @AskDateUnix.setter
    def AskDateUnix(self, value: int):
        ...

    def GetTickTimestamp(self) -> datetime.datetime:
        """Gets the tick timestamp (UTC)"""
        ...


class TradierEvents(System.Object):
    """Events array container."""

    @property
    def Events(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierEvent]:
        ...

    @Events.setter
    def Events(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierEvent]):
        ...

    def __init__(self) -> None:
        ...


class TradierEventContainer(System.Object):
    """Tradier deserialization container for history"""

    @property
    def TradierEvents(self) -> QuantConnect.Brokerages.Tradier.TradierEvents:
        ...

    @TradierEvents.setter
    def TradierEvents(self, value: QuantConnect.Brokerages.Tradier.TradierEvents):
        ...

    def __init__(self) -> None:
        ...


class TradierDividendEvent(QuantConnect.Brokerages.Tradier.TradierEventDetail):
    """Dividend event in history:"""

    def __init__(self) -> None:
        ...


class TradierOrders(System.Object):
    """Order container class"""

    @property
    def Orders(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierOrder]:
        ...

    @Orders.setter
    def Orders(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierOrder]):
        ...

    def __init__(self) -> None:
        ...


class TradierOrdersContainer(System.Object):
    """Order parent class for deserialization"""

    @property
    def Orders(self) -> QuantConnect.Brokerages.Tradier.TradierOrders:
        ...

    @Orders.setter
    def Orders(self, value: QuantConnect.Brokerages.Tradier.TradierOrders):
        ...

    def __init__(self) -> None:
        ...


class TradierOrderDetailedContainer(System.Object):
    """Detailed order parent class"""

    @property
    def DetailedOrder(self) -> QuantConnect.Brokerages.Tradier.TradierOrderDetailed:
        ...

    @DetailedOrder.setter
    def DetailedOrder(self, value: QuantConnect.Brokerages.Tradier.TradierOrderDetailed):
        ...


class TradierBalance(System.Object):
    """Inside "Account" User-account balance information."""

    @property
    def Balances(self) -> QuantConnect.Brokerages.Tradier.TradierBalanceDetails:
        ...

    @Balances.setter
    def Balances(self, value: QuantConnect.Brokerages.Tradier.TradierBalanceDetails):
        ...


class TradierFaultDetail(System.Object):
    """Error code associated with this fault."""

    @property
    def ErrorCode(self) -> str:
        ...

    @ErrorCode.setter
    def ErrorCode(self, value: str):
        ...

    def __init__(self) -> None:
        ...


class TradierFault(System.Object):
    """
    Tradier fault object:
    {"fault":{"faultstring":"Access Token expired","detail":{"errorcode":"keymanagement.service.access_token_expired"}}}
    """

    @property
    def Description(self) -> str:
        ...

    @Description.setter
    def Description(self, value: str):
        ...

    @property
    def Details(self) -> QuantConnect.Brokerages.Tradier.TradierFaultDetail:
        ...

    @Details.setter
    def Details(self, value: QuantConnect.Brokerages.Tradier.TradierFaultDetail):
        ...

    def __init__(self) -> None:
        ...


class TradierFaultContainer(System.Object):
    """Wrapper container for fault:"""

    @property
    def Fault(self) -> QuantConnect.Brokerages.Tradier.TradierFault:
        ...

    @Fault.setter
    def Fault(self, value: QuantConnect.Brokerages.Tradier.TradierFault):
        ...

    def __init__(self) -> None:
        ...


class TradierGainLossClosed(System.Object):
    """Gain loss class"""

    @property
    def ClosedPositions(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierGainLoss]:
        ...

    @ClosedPositions.setter
    def ClosedPositions(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierGainLoss]):
        ...


class TradierGainLossContainer(System.Object):
    """Gain loss parent class for deserialization"""

    @property
    def GainLossClosed(self) -> QuantConnect.Brokerages.Tradier.TradierGainLossClosed:
        ...

    @GainLossClosed.setter
    def GainLossClosed(self, value: QuantConnect.Brokerages.Tradier.TradierGainLossClosed):
        ...

    def __init__(self) -> None:
        ...


class TradierUserContainer(System.Object):
    """Model for a TradierUser returned from the API."""

    @property
    def Profile(self) -> QuantConnect.Brokerages.Tradier.TradierUser:
        ...

    @Profile.setter
    def Profile(self, value: QuantConnect.Brokerages.Tradier.TradierUser):
        ...

    def __init__(self) -> None:
        ...


class TradierPositions(System.Object):
    """Position array container."""

    @property
    def Positions(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierPosition]:
        ...

    @Positions.setter
    def Positions(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Tradier.TradierPosition]):
        ...

    def __init__(self) -> None:
        ...


class TradierPositionsContainer(System.Object):
    """Empty class for deserializing positions held."""

    @property
    def TradierPositions(self) -> QuantConnect.Brokerages.Tradier.TradierPositions:
        ...

    @TradierPositions.setter
    def TradierPositions(self, value: QuantConnect.Brokerages.Tradier.TradierPositions):
        ...

    def __init__(self) -> None:
        ...


class _EventContainer(typing.Generic[QuantConnect_Brokerages_Tradier__EventContainer_Callable, QuantConnect_Brokerages_Tradier__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_Brokerages_Tradier__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_Brokerages_Tradier__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_Brokerages_Tradier__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





import typing

import QuantConnect
import QuantConnect.Algorithm
import QuantConnect.Data
import QuantConnect.Data.UniverseSelection
import QuantConnect.DataLibrary.Tests
import QuantConnect.DataSource
import QuantConnect.Orders
import System.Collections.Generic


class QuiverCongressDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class BitcoinMetadataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """
    Example algorithm using Blockchain Bitcoin Metadata as a source of alpha
    In this algorithm, we're trading the supply-demand of the Bitcoin blockchain services will affect its price
    """

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...


class CoarseTiingoNewsUniverseSelectionAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """
    Example algorithm of a custom universe selection using coarse data and adding TiingoNews
    If conditions are met will add the underlying and trade it
    """

    def CoarseSelectionFunction(self, coarse: System.Collections.Generic.IEnumerable[QuantConnect.Data.UniverseSelection.CoarseFundamental]) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        ...

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...

    def OnSecuritiesChanged(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        ...


class TiingoNewsAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """
    Look for positive and negative words in the news article description
    and trade based on the sum of the sentiment
    """

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class TiingoNewsDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class TradingEconomicsCalendarIndicatorAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This example algorithm shows how to import and use Trading Economics data."""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.TradingEconomicsCalendar) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param data: Trading Economics Calendar object
        """
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.TradingEconomicsIndicator) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param data: Trading Economics Indicator object
        """
        ...


class TradingEconomicsAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Trades on interest rate announcements from data provided by Trading Economics"""

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class TradingEconomicsCalendarDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class CachedFREDDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class QuiverWallStreetBetsDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """
    Quiver Quantitative is a provider of alternative data.
    This algorithm shows how to consume the QuiverWallStreetBets
    """

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class QuiverWallStreetBetsDataDemonstrationAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class BenzingaNewsDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class BenzingaNewsAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """
    Benzinga is a provider of news data. Their news is made in-house
    and covers stock related news such as corporate events.
    """

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...

    def OnSecuritiesChanged(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        ...


class USTreasuryYieldCurveDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Demonstration algorithm showing how to use and access U.S. Treasury yield curve data"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        ...


class USTreasuryYieldCurveRateDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class USTreasuryYieldCurveRateAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class SmartInsiderEventBenchmarkAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class SmartInsiderIntentionsTransactionsDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class SmartInsiderDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm demonstrating usage of SmartInsider data"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    @typing.overload
    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.SmartInsiderTransaction) -> None:
        """
        Insider transaction data will be provided to us here
        
        :param data: Transaction data
        """
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.SmartInsiderIntention) -> None:
        """
        Insider intention data will be provided to us here
        
        :param data: Intention data
        """
        ...


class SmartInsiderTransactionAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def CoarseUniverse(self, coarse: System.Collections.Generic.IEnumerable[QuantConnect.Data.UniverseSelection.CoarseFundamental]) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        ...

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...

    def OnSecuritiesChanged(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        ...


class CachedUSEnergyDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class USEnergyDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class USEnergyInformationAdministrationAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This example algorithm shows how to import and use Tiingo daily prices data."""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...


class VIXCentralContangoDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class QuiverEventsBetaDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class CachedCBOEDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def Initialize(self) -> None:
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class CBOEDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class QuiverWikipediaDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class NasdaqDataLinkDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the Nasdaq Data Link data as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class ExtractAlphaTrueBeatsAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the ExtractAlphaTrueBeats type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class SECReportDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Demonstration algorithm showing how to use and access SEC data"""

    Ticker: str = "AAPL"

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        ...


class SECReportDataDemonstrationAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class SECReport8KAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def CoarseSelector(self, coarse: System.Collections.Generic.IEnumerable[QuantConnect.Data.UniverseSelection.CoarseFundamental]) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        ...

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...

    def OnSecuritiesChanged(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        ...


class SECReportBenchmarkAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This class has no documentation."""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, data: QuantConnect.Data.Slice) -> None:
        ...


class EstimizeDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """This example algorithm shows how to import and use Estimize data types."""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.EstimizeRelease) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param data: EstimizeRelease object containing the stock release data
        """
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.EstimizeEstimate) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param data: EstimizeEstimate object containing the stock release data
        """
        ...

    @typing.overload
    def OnData(self, data: QuantConnect.DataSource.EstimizeConsensus) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param data: EstimizeConsensus object containing the stock release data
        """
        ...


class EstimizeConsensusesEstimatesReleasesDataAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using the custom data type as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...

    def OnOrderEvent(self, orderEvent: QuantConnect.Orders.OrderEvent) -> None:
        """
        Order fill event handler. On an order fill update the resulting information is passed to this method.
        
        :param orderEvent: Order event details containing details of the events
        """
        ...


class CryptoSlamNFTSalesAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Example algorithm using NFT Sales data as a source of alpha"""

    def Initialize(self) -> None:
        """Initialise the data and resolution required, as well as the cash and start-end dates for your algorithm. All algorithms must initialized."""
        ...

    def OnData(self, slice: QuantConnect.Data.Slice) -> None:
        """
        OnData event is the primary entry point for your algorithm. Each new data point will be pumped in here.
        
        :param slice: Slice object keyed by symbol containing the stock data
        """
        ...





import typing

import QuantConnect.Brokerages.Binance.Messages
import System

JsonConverter = typing.Any


class BalanceEntry(System.Object):
    """This class has no documentation."""

    @property
    def Asset(self) -> str:
        ...

    @Asset.setter
    def Asset(self, value: str):
        ...

    @property
    def Free(self) -> float:
        ...

    @Free.setter
    def Free(self, value: float):
        ...

    @property
    def Locked(self) -> float:
        ...

    @Locked.setter
    def Locked(self, value: float):
        ...

    @property
    def Amount(self) -> float:
        ...


class AccountInformation(System.Object):
    """This class has no documentation."""

    @property
    def Balances(self) -> typing.List[QuantConnect.Brokerages.Binance.Messages.BalanceEntry]:
        ...

    @Balances.setter
    def Balances(self, value: typing.List[QuantConnect.Brokerages.Binance.Messages.BalanceEntry]):
        ...


class SpotBalance(QuantConnect.Brokerages.Binance.Messages.BalanceEntry):
    """This class has no documentation."""

    @property
    def Amount(self) -> float:
        ...


class MarginBalance(QuantConnect.Brokerages.Binance.Messages.BalanceEntry):
    """This class has no documentation."""

    @property
    def Borrowed(self) -> float:
        ...

    @Borrowed.setter
    def Borrowed(self, value: float):
        ...

    @property
    def NetAsset(self) -> float:
        ...

    @NetAsset.setter
    def NetAsset(self, value: float):
        ...

    @property
    def Amount(self) -> float:
        ...


class PriceTicker(System.Object):
    """This class has no documentation."""

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...


class Order(System.Object):
    """This class has no documentation."""

    @property
    def Id(self) -> str:
        ...

    @Id.setter
    def Id(self, value: str):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def StopPrice(self) -> float:
        ...

    @StopPrice.setter
    def StopPrice(self, value: float):
        ...

    @property
    def OriginalAmount(self) -> float:
        ...

    @OriginalAmount.setter
    def OriginalAmount(self, value: float):
        ...

    @property
    def ExecutedAmount(self) -> float:
        ...

    @ExecutedAmount.setter
    def ExecutedAmount(self, value: float):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def Type(self) -> str:
        ...

    @Type.setter
    def Type(self, value: str):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Quantity(self) -> float:
        ...


class OpenOrder(QuantConnect.Brokerages.Binance.Messages.Order):
    """This class has no documentation."""

    @property
    def Time(self) -> int:
        ...

    @Time.setter
    def Time(self, value: int):
        ...


class NewOrder(QuantConnect.Brokerages.Binance.Messages.Order):
    """This class has no documentation."""

    @property
    def TransactionTime(self) -> int:
        ...

    @TransactionTime.setter
    def TransactionTime(self, value: int):
        ...


class EventType(System.Enum):
    """This class has no documentation."""

    # Cannot convert to Python: None = 0

    OrderBook = 1

    Trade = 2

    Execution = 3


class ErrorMessage(System.Object):
    """This class has no documentation."""

    @property
    def Code(self) -> int:
        ...

    @Code.setter
    def Code(self, value: int):
        ...

    @property
    def Message(self) -> str:
        ...

    @Message.setter
    def Message(self, value: str):
        ...


class BestBidAskQuote(System.Object):
    """This class has no documentation."""

    @property
    def OrderBookUpdateId(self) -> int:
        ...

    @OrderBookUpdateId.setter
    def OrderBookUpdateId(self, value: int):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...

    @property
    def BestBidPrice(self) -> float:
        ...

    @BestBidPrice.setter
    def BestBidPrice(self, value: float):
        ...

    @property
    def BestBidSize(self) -> float:
        ...

    @BestBidSize.setter
    def BestBidSize(self, value: float):
        ...

    @property
    def BestAskPrice(self) -> float:
        ...

    @BestAskPrice.setter
    def BestAskPrice(self, value: float):
        ...

    @property
    def BestAskSize(self) -> float:
        ...

    @BestAskSize.setter
    def BestAskSize(self, value: float):
        ...


class BaseMessage(System.Object):
    """This class has no documentation."""

    # Cannot convert property @Event to Python

    @property
    def EventName(self) -> str:
        ...

    @EventName.setter
    def EventName(self, value: str):
        ...

    @property
    def Time(self) -> int:
        ...

    @Time.setter
    def Time(self, value: int):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...


class Trade(QuantConnect.Brokerages.Binance.Messages.BaseMessage):
    """This class has no documentation."""

    # Cannot convert property @Event to Python

    @property
    def Time(self) -> int:
        ...

    @Time.setter
    def Time(self, value: int):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Quantity(self) -> float:
        ...

    @Quantity.setter
    def Quantity(self, value: float):
        ...


class Execution(QuantConnect.Brokerages.Binance.Messages.BaseMessage):
    """This class has no documentation."""

    # Cannot convert property @Event to Python

    @property
    def OrderId(self) -> str:
        ...

    @OrderId.setter
    def OrderId(self, value: str):
        ...

    @property
    def TradeId(self) -> str:
        ...

    @TradeId.setter
    def TradeId(self, value: str):
        ...

    @property
    def Ignore(self) -> str:
        ...

    @Ignore.setter
    def Ignore(self, value: str):
        ...

    @property
    def ExecutionType(self) -> str:
        ...

    @ExecutionType.setter
    def ExecutionType(self, value: str):
        ...

    @property
    def OrderStatus(self) -> str:
        ...

    @OrderStatus.setter
    def OrderStatus(self, value: str):
        ...

    @property
    def TransactionTime(self) -> int:
        ...

    @TransactionTime.setter
    def TransactionTime(self, value: int):
        ...

    @property
    def LastExecutedPrice(self) -> float:
        ...

    @LastExecutedPrice.setter
    def LastExecutedPrice(self, value: float):
        ...

    @property
    def LastExecutedQuantity(self) -> float:
        ...

    @LastExecutedQuantity.setter
    def LastExecutedQuantity(self, value: float):
        ...

    @property
    def Side(self) -> str:
        ...

    @Side.setter
    def Side(self, value: str):
        ...

    @property
    def Fee(self) -> float:
        ...

    @Fee.setter
    def Fee(self, value: float):
        ...

    @property
    def FeeCurrency(self) -> str:
        ...

    @FeeCurrency.setter
    def FeeCurrency(self, value: str):
        ...

    @property
    def Direction(self) -> int:
        """This property contains the int value of a member of the QuantConnect.Orders.OrderDirection enum."""
        ...


class Kline(System.Object):
    """This class has no documentation."""

    @property
    def OpenTime(self) -> int:
        ...

    @property
    def Open(self) -> float:
        ...

    @property
    def Close(self) -> float:
        ...

    @property
    def High(self) -> float:
        ...

    @property
    def Low(self) -> float:
        ...

    @property
    def Volume(self) -> float:
        ...

    @typing.overload
    def __init__(self) -> None:
        ...

    @typing.overload
    def __init__(self, msts: int, close: float) -> None:
        ...

    @typing.overload
    def __init__(self, entries: typing.List[System.Object]) -> None:
        ...


class SpotAccountConverter(JsonConverter):
    """
    Deserializes Spot Account data
    https://binance-docs.github.io/apidocs/spot/en/#account-information-user_data
    """

    def CanConvert(self, objectType: typing.Type) -> bool:
        ...

    def ReadJson(self, reader: typing.Any, objectType: typing.Type, existingValue: typing.Any, serializer: typing.Any) -> System.Object:
        """
        Reads the JSON representation of the spot account data and asset balances.
        
        :param reader: The Newtonsoft.Json.JsonReader to read from.
        :param objectType: Type of the object.
        :param existingValue: The existing value of object being read.
        :param serializer: The calling serializer.
        :returns: The object value.
        """
        ...

    def WriteJson(self, writer: typing.Any, value: typing.Any, serializer: typing.Any) -> None:
        ...


class MarginAccountConverter(JsonConverter):
    """
    Deserializes cross margin Account data
    https://binance-docs.github.io/apidocs/spot/en/#query-cross-margin-account-details-user_data
    """

    def CanConvert(self, objectType: typing.Type) -> bool:
        ...

    def ReadJson(self, reader: typing.Any, objectType: typing.Type, existingValue: typing.Any, serializer: typing.Any) -> System.Object:
        """
        Reads the JSON representation of the margin account data and asset balances.
        
        :param reader: The Newtonsoft.Json.JsonReader to read from.
        :param objectType: Type of the object.
        :param existingValue: The existing value of object being read.
        :param serializer: The calling serializer.
        :returns: The object value.
        """
        ...

    def WriteJson(self, writer: typing.Any, value: typing.Any, serializer: typing.Any) -> None:
        ...





import datetime
import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Brokerages.Bitfinex
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Concurrent
import System.Collections.Generic


class BitfinexBrokerage(QuantConnect.Brokerages.BaseWebsocketsBrokerage, QuantConnect.Interfaces.IDataQueueHandler):
    """Utility methods for Bitfinex brokerage"""

    @property
    def UnixEpoch(self) -> datetime.datetime:
        """Unix Epoch"""
        ...

    ApiKeyHeader: str = "bfx-apikey"
    """ApiKey Header"""

    NonceHeader: str = "bfx-nonce"
    """Nonce Header"""

    SignatureHeader: str = "bfx-signature"
    """Signature Header"""

    @property
    def TickLocker(self) -> System.Object:
        """Locking object for the Ticks list in the data queue handler"""
        ...

    @property
    def IsConnected(self) -> bool:
        ...

    @typing.overload
    def __init__(self) -> None:
        """Constructor for brokerage"""
        ...

    @typing.overload
    def __init__(self, apiKey: str, apiSecret: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Constructor for brokerage
        
        :param apiKey: api key
        :param apiSecret: api secret
        :param algorithm: the algorithm instance is required to retrieve account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: consolidate ticks
        :param job: The live job packet
        """
        ...

    @typing.overload
    def __init__(self, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Constructor for brokerage
        
        :param websocket: instance of websockets client
        :param restClient: instance of rest client
        :param apiKey: api key
        :param apiSecret: api secret
        :param algorithm: the algorithm instance is required to retrieve account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: consolidate ticks
        :param job: The live job packet
        """
        ...

    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Cancels the order with the specified ID
        
        :param order: The order to cancel
        :returns: True if the request was submitted for cancellation, false otherwise.
        """
        ...

    def Disconnect(self) -> None:
        """Closes the websockets connection"""
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """Gets all open positions"""
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """Gets the total account cash balance for specified account type"""
        ...

    def GetHistory(self, request: QuantConnect.Data.HistoryRequest) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.BaseData]:
        """
        Gets the history for the requested security
        
        :param request: The historical data request
        :returns: An enumerable of bars covering the span specified in the request.
        """
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """Gets all orders not yet closed"""
        ...

    def GetSubscribed(self) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        """
        Should be empty. BrokerageMultiWebSocketSubscriptionManager manages each BitfinexWebSocketWrapper individually
        
        This method is protected.
        """
        ...

    def GetTick(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> QuantConnect.Data.Market.Tick:
        """Provides the current best bid and ask"""
        ...

    @typing.overload
    def OnMessage(self, sender: typing.Any, e: QuantConnect.Brokerages.WebSocketMessage) -> None:
        """
        Wss message handler
        
        This method is protected.
        """
        ...

    @typing.overload
    def OnMessage(self, e: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Places a new order and assigns a new broker ID to the order
        
        :param order: The order to be placed
        :returns: True if the request for a new order has been placed, false otherwise.
        """
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        ...

    @typing.overload
    def Subscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> bool:
        """
        Should be empty, Bitfinex brokerage manages his public channels including subscribe/unsubscribe/reconnect methods using BrokerageMultiWebSocketSubscriptionManager
        Not used in master
        
        This method is protected.
        """
        ...

    @typing.overload
    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: typing.Callable[[System.Object, System.EventArgs], None]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def SubscribeAuth(self) -> None:
        """Subscribes to the authenticated channels (using an single streaming channel)"""
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Updates the order with the same id
        
        :param order: The new order information
        :returns: True if the request was made for the order to be updated, false otherwise.
        """
        ...


class BitfinexWebSocketWrapper(QuantConnect.Brokerages.WebSocketClientWrapper):
    """Wrapper class for a Bitfinex websocket connection"""

    @property
    def ConnectionId(self) -> str:
        """The unique Id for the connection"""
        ...

    @property
    def ConnectionHandler(self) -> QuantConnect.Brokerages.IConnectionHandler:
        """The handler for the connection"""
        ...

    def __init__(self, connectionHandler: QuantConnect.Brokerages.IConnectionHandler) -> None:
        """Initializes a new instance of the BitfinexWebSocketWrapper class."""
        ...


class BitfinexBrokerageFactory(QuantConnect.Brokerages.BrokerageFactory):
    """Factory method to create Bitfinex Websockets brokerage"""

    @property
    def BrokerageData(self) -> System.Collections.Generic.Dictionary[str, str]:
        """provides brokerage connection data"""
        ...

    def __init__(self) -> None:
        """Factory constructor"""
        ...

    def CreateBrokerage(self, job: QuantConnect.Packets.LiveNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Interfaces.IBrokerage:
        """Create the Brokerage instance"""
        ...

    def Dispose(self) -> None:
        """Not required"""
        ...

    def GetBrokerageModel(self, orderProvider: QuantConnect.Securities.IOrderProvider) -> QuantConnect.Brokerages.IBrokerageModel:
        """
        The brokerage model
        
        :param orderProvider: The order provider
        """
        ...


class BitfinexWebSocketChannels(System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Data.Channel]):
    """Contains the channel mappings for a WebSocket connection"""

    def Contains(self, channel: QuantConnect.Data.Channel) -> bool:
        """
        Determines whether the dictionary contains a specific channel.
        
        :param channel: The channel
        :returns: true if the channel was found.
        """
        ...

    def GetChannelId(self, channel: QuantConnect.Data.Channel) -> int:
        """
        Returns the channel id for the given channel.
        
        :param channel: The channel
        :returns: The channel id.
        """
        ...


